'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _Pair = require('./Pair');

var _Pair2 = _interopRequireDefault(_Pair);

var _Types = require('../types/Types');

var _Types2 = _interopRequireDefault(_Types);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * Analyzer
 *
 * Analyzes parsed array and generates pairs array
 *
 * @static
 * @internal
 */
var Analyzer = function () {
  function Analyzer() {
    _classCallCheck(this, Analyzer);
  }

  _createClass(Analyzer, null, [{
    key: 'analyze',


    /**
     * Analyze parsed array
     *
     * @param {Array<string>} array parsed array
     * @return {Array<Pair>} array analyzed array
     * @static
     */


    /**
     * Invalid value
     *
     * @param {int} MODE_VALUE value mode
     * @static
     */


    /**
     * letter
     *
     * @param {RegExp} leter regular expression for leter
     * @static
     */
    value: function analyze(array) {
      return Analyzer.complete(array);
    }

    /**
     * Complete parsed array
     *
     * Tries to complete left and right missing units or values
     *
     * @param {Array<string>} array parsed array
     * @return {Array<Pair>} array analyzed array
     * @static
     */


    /**
     * Invalid unit
     *
     * @param {int} MODE_UNIT unit mode
     * @static
     */


    /**
     * Invalid mode
     *
     * @param {int} MODE_INVALID invalid mode
     * @static
     */

    /**
     * digit
     *
     * @param {RegExp} digit regular expression for digit
     * @static
     */

  }, {
    key: 'complete',
    value: function complete(array) {
      var modified = Analyzer.normalize(array);

      var skipped = false;
      for (var i = 0; i < modified.length; ++i) {
        var pair = modified[i];
        if (pair.unit && skipped) {
          skipped = false;

          for (var j = i - 1; j >= 0; --j) {
            var leftPair = modified[j];
            var rightPair = modified[j + 1];
            if (!leftPair.unit) {
              leftPair.unit = _Types2.default.greater(rightPair.unit);
              if (!leftPair.unit) {
                modified.splice(j, 1);
                --i;
              }
            } else {
              break;
            }
          }
        } else if (!pair.unit) {
          if (i > 0) {
            var _leftPair = modified[i - 1];
            if (_leftPair.unit) {
              var lesser = _Types2.default.lesser(_leftPair.unit);
              if (lesser) {
                pair.unit = lesser;
              } else {
                skipped = true;
              }
            } else {
              skipped = true;
            }
          } else {
            skipped = true;
          }
        }
      }

      while (modified.length > 0 && !modified[modified.length - 1].unit) {
        modified.splice(modified.length - 1, 1);
      }

      return modified;
    }

    /**
     * Normalizes parsed array
     *
     * Removes missing or invalid value unit couples
     *
     * @param {Array<string>} array parsed array
     * @return {Array<Pair>} array analyzed array
     * @static
     */

  }, {
    key: 'normalize',
    value: function normalize(array) {
      var modified = Analyzer.group(array);

      for (var i = 0; i < modified.length; ++i) {
        var pair = modified[i];
        if (pair.unit) {
          var validated = _Types2.default.validate(pair.unit);
          if (validated !== undefined) {
            pair.unit = validated;
          } else {
            modified.splice(i, 1);
            --i;
          }
        }
      }

      return modified;
    }

    /**
     * Groups parsed array
     *
     * Groups value units into pairs
     *
     * @param {Array<string>} array parsed array
     * @return {Array<Pair>} array analyzed array
     * @static
     */

  }, {
    key: 'group',
    value: function group(array) {
      var length = array.length;

      var parts = [];
      var i = void 0;
      var symbol = void 0;
      var current = void 0;
      var mode = void 0;

      for (i = 0; i < length; ++i) {
        symbol = array[i];

        if (current === undefined) {
          current = new _Pair2.default();
        }

        if (Analyzer.digit.test(symbol)) {
          mode = Analyzer.MODE_VALUE;
        } else if (Analyzer.letter.test(symbol)) {
          mode = Analyzer.MODE_UNIT;
        } else {
          mode = Analyzer.MODE_INVALID;
        }

        if (mode === Analyzer.MODE_INVALID) {
          //
        } else if (mode === Analyzer.MODE_VALUE) {
          if (current.value === undefined) {
            current.value = parseInt(symbol);
          } else {
            parts.push(current);
            current = new _Pair2.default(parseInt(symbol));
          }
        } else if (mode === Analyzer.MODE_UNIT) {
          current.unit = symbol;

          if (current.value === undefined) {
            current.value = 1;
          }
        }

        if (current.unit !== undefined) {
          parts.push(current);
          current = undefined;
        }
      }

      if (current !== undefined) {
        parts.push(current);
        current = undefined;
      }

      return parts;
    }
  }]);

  return Analyzer;
}();

Analyzer.digit = /[0-9]+/;
Analyzer.letter = /[a-zA-Z]+/;
Analyzer.MODE_INVALID = 0;
Analyzer.MODE_VALUE = 1;
Analyzer.MODE_UNIT = 2;
exports.default = Analyzer;